#ifndef ESCENA_H
#define ESCENA_H

#define GL_GLEXT_PROTOTYPES 1

#include <stdio.h>
#include <Common.h>
#include <QGLShaderProgram>

#include <cmath>
#include <iostream>
#include <stdlib.h>

#include <QtOpenGL>
#include <QGLWidget>
#include <QtGui>

#include <list>
#include <vector>
#include <string>


#include <camera.h>
#include <objecte.h>
#include <cub.h>
#include <llum.h>
#include <volumen.h>
#include <lut.h>
using namespace std;

class escena : public QGLWidget
{
    Q_OBJECT

public:

    escena();
    ~escena();

    int xRotation();
    int yRotation();
    int zRotation();

    void refresh();
    void guardar();
    void initializeGL();
    void setLut(const char *nombreFichero);
    void setVolumen(const char *nombreFichero, int anchura, int altura, int profundidad, QString tipoUnsigned);
    void addVolumen(volumen *v);

    void CalculCamera(bool retallat, bool centrat);
    void CapsaMinCont3DEscena();
    void CameraToGL();
    void raycastingCPU(float opacitat, float saturacio, float sampling);
    void setRayCastingFlag(int mode);
    void startRaycasting();

    void setOpacitat(float op);
    void setSaturacio(float sat);
    void setSampling(float samp);
    void setColorMode(int mode);

    void setOneStepRaycastingMode(bool mode);
    void setMipMode(bool mode);

    void setRampColorTable(QVector<unsigned int> qrgbcolors);
    void setInitialLUT(QVector<unsigned int> qrgbcolors);

    void eliminaCub();

    static const int NumVerticesCUB = 4*6; //4 vertex * 6 cares
    vec3 tex_coords3D[NumVerticesCUB];
    int Index;

public slots:
    void Pan(int dx, int dy);
    void Zoom(int positiu);
     void setXRotation(int angle);
     void setYRotation(int angle);

signals:

    void RotationChanged(int dx, int dy);
    void PanChanged(int dx, int dy);
    void ZoomChanged(int positiu);
    void xRotationChanged(int angle);
    void yRotationChanged(int angle);

protected:

    void paintGL();
    void resizeGL(int width, int height);
    int pointAtPosition(const QPoint &pos);
    virtual void mousePressEvent(QMouseEvent *event);
    virtual void mouseMoveEvent(QMouseEvent *event);

private slots:

private:

    void normalizeAngle(int *angle);

    void InitShader( const char* vertexShaderFile,
                       const char* fragmentShaderFile );

    void initShadersGPU();
    void initBackfaceBuffers();
    void vertex(float x, float y, float z);
    void drawQuads(float x, float y, float z);
    void renderBackface();
    void textures2DtoGPU(GLuint texName1);
    void creaTextura3D();
    void dadesRCtoGPU();
    void texturaLutToGPU();
    GLubyte* getLutBytes(lut *lutActual);

    GLuint BFframebuffer;
    GLuint backface;
    GLuint BFrenderbuffer;
    GLuint texturaVolum;
    GLuint texturaLut;

    QPoint lastPos;

    Camera camera;
    Capsa3D capsaMinima;

    mat4 modelView;
    mat4 projection;

    float opacitat, sampling;
    int saturacio;
    int colorMode;
    bool oneStepMode, mipMode;
    float anchVolum, altVolum, profVolum;

    QVector<unsigned int> colorTable;

    Objecte *obj;
    Cub *cub;
    lut *LUT;
    Llum *llum;

    QWidget *w;
    QLineEdit *nombreEditado;

    QGLShaderProgram *program;
};

#endif // escena_H
