#ifndef MAINWINDOW_H
#define MAINWINDOW_H


#include <textureWidget.h>
#include <escena.h>
#include <QColorRampEditor.h>
#include <QMainWindow>


namespace Ui {
    class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

private slots:

    void rcCPU();
    void rcGPU();
    void setOpacitat(int);
    void setSaturacio(int);
    void setSampling(int);
    void setOpacitatGPU(int);
    void setSaturacioGPU(int);
    void setSamplingGPU(int);
    void onRampChanged();

    void setGsMode();
    void setLutMode();
    void setPhongMode();
    void setMipMode();

    void setOneStepRaycastingGPU();

    void cargarVolumen(volumen *v);
    void canvioDeAtributos();
    void aplicarCambios();

private:
    void createActions();
    void createMenus();
    void initRcInteractiveComponents();

    // Widgets i layouts
    Ui::MainWindow *ui;
    QWidget *centralWidget;
    QGridLayout *centralLayout;
    QStackedLayout *layoutStack;
    QStackedLayout *layoutStack2;
    QStackedLayout *layoutStack3;
    QWidget *window;
    QDialog *dialog;

    QSpinBox * integerSpinBox1;
    QSpinBox * integerSpinBox2;
    QSpinBox * integerSpinBox3;
    QComboBox * unsig;
    QDialogButtonBox *buttonBox;
    QSlider *umbralOpacitat;
    QSlider *umbralSaturacio;
    QSlider *umbralSampling;
    QSlider *umbralOpacitatGPU;
    QSlider *umbralSaturacioGPU;
    QSlider *umbralSamplingGPU;
    QLabel *opacitatLabel;
    QLabel *saturacioLabel;
    QLabel *samplingLabel;
    QLabel *opacitatLabelGPU;
    QLabel *saturacioLabelGPU;
    QLabel *samplingLabelGPU;
    QRadioButton *gsCheck;
    QRadioButton *lutCheck;
    QRadioButton *phongCheck;
    QCheckBox *oneStepCheck;
    QRadioButton *MipCheck;
    QCheckBox *RampCheck;

    QColorRampEditor* ramph;

    QScrollArea *glWidgetArea;
    QScrollArea *glWidgetArea2;
    QScrollArea *glWidgetArea3;

    textureWidget *texW;
    escena *esc;

    // Accions associades als menus
    QAction *exitAct;
    QAction *addCubo;
    QAction *addEsfera;
    QAction *addTauler;
    QAction *addDama;
    QAction *addVolumen;
    QAction *openDataSet;

    QAction *CPU;
    QAction *GPU;

    // Menus principals
    QMenu *fileMenu;
    QMenu *rcMenu;

    //Raycasting values
    float opacitat, saturacio, sampling, opacitatGPU, samplingGPU;
    int saturacioGPU;
    static const int gsMode = 0;
    static const int lutMode = 1;
    static const int phongMode = 2;
    static const int mipMode = 3;

    bool volum;

};

#endif // MAINWINDOW_H
